<?php
########################################################
# 				RADIOBOSS REQUEST SONG                 #
#             Version : 1.5 - 21/11/2024               #
#                        AUTHOR                        #
#           Cedric Huraux / c.huraux@yahoo.fr          #
########################################################

#### SETTINGS #######
#FORM
$Title ="Choose your song";
$Description ="Choose your favorite song from our playlist";
$FormTitle1 ="Search for a song :";
$FormHolder1 ="Search...";
$FormTitle2 ="Add a message :";
$FormHolder2 ="Enter your message here";
$FormButton ="Submit";
$FormColArtist ="Artist";
$FormColTitle ="Title";
$FormAlertBoxOK ="Request received";
$FormAlertBoxError ="Error during processing";
$DelayTimeBeforeSubmitAgain = 3600000; // Delay before submit form again in milliseconds (3600000 = 1H in milliseconds)
$FormAlertBoxDelay ="Please wait 1 hour before resubmitting a request";
$FormNumberSongByPage = 30; // Songs by page
$FormNextPageButton ="Next";
$FormPreviousPageButton ="Previous";
$FormSubmitBoxError ="Please select a file and enter a message.";

#RADIOBOSS
$RadiobossUrl ="http://ipradioboss:port/"; // RadioBoss URL example : http://ipradioboss:port/
$RadiobossPassword ="Password"; // RadioBoss API Password
$RadiobossLibrary ="Music"; // RadioBoss Music library name
$RadiobossError ="This service is not available at the moment"; // RadioBoss error message if not online

#### END SETTINGS #######

$RadiobossLibraryURL = "$RadiobossUrl?pass=$RadiobossPassword&action=library&filename=$RadiobossLibrary";
?>
<!DOCTYPE html>
<html lang="fr">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>SONG REQUEST</title>
	<link rel="stylesheet" type="text/css" href="request_style.css" />
</head>
<body>
    <div class="container">
        <h3><?php echo $Title; ?></h3>
        <h4><?php echo $Description; ?></h4>
        <span><?php echo $FormTitle1; ?></span>
        <input type="text" id="searchInput" placeholder="<?php echo $FormHolder1; ?>" onkeyup="filterTable()">
        <span><?php echo $FormTitle2; ?></span>
        <input type="text" id="messageInput" placeholder="<?php echo $FormHolder2; ?>">
        <form id="hiddenForm" method="get" action="<?php echo $RadiobossUrl; ?>">
            <input type="hidden" name="pass" value="<?php echo $RadiobossPassword; ?>">
            <input type="hidden" name="action" value="songrequest">
            <input type="hidden" name="message" id="hiddenMessage">
            <input type="hidden" name="filename" id="hiddenFilename">
        </form>
        <button onclick="sendFilenames()"><?php echo $FormButton; ?></button>

        <div id="musicTableContainer"></div>
        <div class="pagination" id="paginationControls"></div>

        <script>
            let allRows = [];
            const rowsPerPage = <?php echo $FormNumberSongByPage; ?>;
            let currentPage = 1;
            let filteredRows = [];
            
            document.addEventListener('DOMContentLoaded', () => {
                initializeTable();
            });

            function initializeTable() {
                const table = document.getElementById('musicTable');
                const tr = table.getElementsByTagName('tr');
                allRows = Array.from(tr).slice(1); // Exclude header row
                filterTable(); // initialize filtering
            }
            
            function filterTable() {
                const input = document.getElementById('searchInput');
                const filter = input.value.toUpperCase();
                filteredRows = allRows.filter(row => {
                    const td = row.getElementsByTagName('td');
                    for (let i = 1; i < td.length; i++) {
                        if (td[i]) {
                            const txtValue = td[i].textContent || td[i].innerText;
                            if (txtValue.toUpperCase().indexOf(filter) > -1) {
                                return true;
                            }
                        }
                    }
                    return false;
                });

                currentPage = 1;
                renderTable();
                setupPagination();
            }

			function setupPagination() {
				const totalPages = Math.ceil(filteredRows.length / rowsPerPage);
				const paginationControls = document.getElementById('paginationControls');
				paginationControls.innerHTML = '';

				// Créez le bouton "Précédente"
				if (currentPage > 1) {
					const prevButton = document.createElement('button');
					prevButton.innerText = '<?php echo $FormPreviousPageButton; ?>';
					prevButton.addEventListener('click', () => {
						if (currentPage > 1) {
							currentPage--; // Diminue le numéro de page
							renderTable(); // Rendu de la table pour la page actuelle
							setupPagination(); // Actualiser la pagination
						}
					});
					paginationControls.appendChild(prevButton);
				}
				
				// Créez le bouton "Suivante"
				if (currentPage < totalPages) {
					const nextButton = document.createElement('button');
					nextButton.innerText = '<?php echo $FormNextPageButton; ?>';
					nextButton.addEventListener('click', () => {
						if (currentPage < totalPages) {
							currentPage++; // Augmente le numéro de page
							renderTable(); // Rendu de la table pour la page actuelle
							setupPagination(); // Actualiser la pagination
						}
					});
					paginationControls.appendChild(nextButton);
				}
			}

            function renderTable() {
                const tableContainer = document.getElementById('musicTableContainer');
                const start = (currentPage - 1) * rowsPerPage;
                const end = start + rowsPerPage;
                const pagedRows = filteredRows.slice(start, end);

                const table = document.createElement('table');
                table.id = 'musicTable';
                table.innerHTML = `
                    <tr>
                        <th></th>
                        <th><?php echo $FormColArtist; ?></th>
                        <th><?php echo $FormColTitle; ?></th>
                    </tr>
                `;

                pagedRows.forEach(row => {
                    table.appendChild(row.cloneNode(true));
                });

                tableContainer.innerHTML = '';
                tableContainer.appendChild(table);
            }

            function handleCheckboxClick(currentCheckbox) {
                const checkboxes = document.querySelectorAll('input[type="checkbox"]');
                checkboxes.forEach(checkbox => {
                    if (checkbox !== currentCheckbox) {
                        checkbox.checked = false;
                    }
                });
            }

            function sendFilenames() {
                const checkbox = document.querySelector('input[type="checkbox"]:checked');
                const messageInput = document.getElementById('messageInput').value.trim();
                
                const lastSubmittedTime = localStorage.getItem('lastSubmittedTime');
                const currentTime = new Date().getTime();
                const delay = <?php echo $DelayTimeBeforeSubmitAgain; ?>;
                
                if (lastSubmittedTime && currentTime - lastSubmittedTime < delay) {
                    alert('<?php echo $FormAlertBoxDelay; ?>');
                    return;
                }

                if (checkbox) {
                    const filename = checkbox.value;
                    const message = messageInput.trim();

                    // Create the API URL
                    const apiUrl = `request_submit.php?filename=${encodeURIComponent(filename)}&message=${encodeURIComponent(message)}`;

                    fetch(apiUrl)
                        .then(response => response.text())
                        .then(result => {
                            if (result === 'OK') {
                                alert('<?php echo $FormAlertBoxOK; ?>');
                                localStorage.setItem('lastSubmittedTime', currentTime);
                            } else {
                                alert('<?php echo $FormAlertBoxError; ?>');
                            }
                        })
                        .catch(error => {
                            alert('Network error during the request.');
                        });
                } else {
                    alert('<?php echo $FormSubmitBoxError; ?>');
                }
            }
        </script>

        <?php
        function checkWebsiteStatus($url) {
            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_NOBODY, true);
            curl_setopt($ch, CURLOPT_TIMEOUT, 2);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_exec($ch);
            $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);
            return ($httpCode >= 200 && $httpCode < 400);
        }

        function formatXmlToHtmlTable($url) {
            $xmlContent = file_get_contents($url);
            if ($xmlContent === false) {
                return "Erreur lors de la récupération du contenu XML.";
            }
            $xml = simplexml_load_string($xmlContent);
            if ($xml === false) {
                return "Erreur lors du chargement du XML.";
            }
            $htmlOutput = '
                <table id="musicTable" style="display: none;">
                <tr>
                    <th></th>
                    <th>Artiste</th>
                    <th>Titre</th>
                </tr>';
            if (isset($xml->Track)) {
                foreach ($xml->Track as $track) {
                    $attributes = $track->attributes();
                    $filename = htmlspecialchars(trim((string)$attributes->filename));
                    $title = htmlspecialchars(trim((string)$attributes->title));
                    $artist = htmlspecialchars(trim((string)$attributes->artist));

                    if (!empty($filename) || !empty($title) || !empty($artist)) {
                        $htmlOutput .= '<tr>';
                        $htmlOutput .= '<td><input type="checkbox" value="' . $filename . '" onclick="handleCheckboxClick(this)"></td>';
                        $htmlOutput .= '<td>' . $artist . '</td>';
                        $htmlOutput .= '<td>' . $title . '</td>';
                        $htmlOutput .= '</tr>';
                    }
                }
            }
            $htmlOutput .= '</table>';
            return $htmlOutput;
        }

        if (checkWebsiteStatus($RadiobossLibraryURL)) {
            echo formatXmlToHtmlTable($RadiobossLibraryURL);
        } else {
            echo "<h3>$RadiobossError</h3>";
            exit;
        }
        ?>
    </div>
</body>
</html>